import { Form, Checkbox as Checkbox$1, DatePicker as DatePicker$1, Input as Input$1, Radio as Radio$1, TimePicker as TimePicker$1, Upload as Upload$1, AutoComplete as AutoComplete$1, Cascader as Cascader$1, InputNumber as InputNumber$1, Mentions as Mentions$1, Rate as Rate$1, Select as Select$1, Slider as Slider$1, Switch as Switch$1, Transfer as Transfer$1, TreeSelect as TreeSelect$1 } from 'antd';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import { forwardRef, useRef, useEffect } from 'react';

const { Item: Item$2 } = Form;
const miscTypeMap = {
    boolean: { type: 'boolean' },
    url: { type: 'url' },
    email: { type: 'email' },
};
const numTypeMap = {
    string: { type: 'string', whitespace: true },
    pureString: { type: 'string' },
    number: { type: 'number' },
    array: { type: 'array' },
};
const getRules = (rules) => {
    const ruleList = [];
    let numTypeRule = null;
    let numValRules = [];
    rules.forEach((rule) => {
        if (typeof rule !== 'string') {
            ruleList.push(rule);
            return;
        }
        if (rule === 'required') {
            ruleList.push({ required: true });
            return;
        }
        if (rule in miscTypeMap) {
            ruleList.push(miscTypeMap[rule]);
            return;
        }
        if (rule in numTypeMap) {
            numTypeRule = numTypeMap[rule];
            return;
        }
        const [key, val] = rule.split('=');
        if (val === undefined) {
            return;
        }
        if (key === 'required') {
            ruleList.push({ required: true, message: val });
            return;
        }
        if (key === 'len' || key === 'min' || key === 'max') {
            numValRules.push({ [key]: +val });
            return;
        }
    });
    if (numTypeRule && numValRules.length > 0) {
        ruleList.push(...numValRules.map((obj) => (Object.assign(Object.assign({}, numTypeRule), obj))));
    }
    else if (numTypeRule) {
        ruleList.push(numTypeRule);
    }
    else if (numValRules.length > 0) {
        ruleList.push(...numValRules);
    }
    return ruleList;
};
const replaceMap = {
    selfClass: 'className',
    selfStyle: 'style',
    selfName: 'name',
    selfTooltip: 'tooltip',
};
const formItemKeys = [
    'colon',
    'dependencies',
    'extra',
    'getValueFromEvent',
    'getValueProps',
    'hasFeedback',
    'help',
    'hidden',
    'htmlFor',
    'initialValue',
    'label',
    'labelAlign',
    'labelCol',
    'messageVariables',
    'name',
    'normalize',
    'noStyle',
    'preserve',
    'required',
    'rules',
    'shouldUpdate',
    'tooltip',
    'trigger',
    'validateFirst',
    'validateStatus',
    'validateTrigger',
    'valuePropName',
    'wrapperCol',
    // extra
    'className',
    'style',
];
const formItemProps = formItemKeys.reduce((obj, key) => {
    obj[key] = true;
    return obj;
}, {});
const create = (Field, getDefaultFieldProps) => {
    const ItemField = forwardRef((props, ref) => {
        const itemProps = {};
        const fieldProps = {};
        // split props
        if (props) {
            Object.keys(props).forEach((key) => {
                const val = props[key];
                if (key in formItemProps) {
                    const itemKey = key;
                    itemProps[itemKey] = key === 'rules' && val ? getRules(val) : val;
                }
                else {
                    const fieldKey = replaceMap[key] || key;
                    fieldProps[fieldKey] = val;
                }
            });
        }
        // default field props
        if (getDefaultFieldProps) {
            const extraFieldProps = getDefaultFieldProps(fieldProps);
            Object.keys(extraFieldProps).forEach((key) => {
                if (!(key in fieldProps)) {
                    fieldProps[key] = extraFieldProps[key];
                }
            });
        }
        const RawField = Field;
        return (jsx(Item$2, Object.assign({}, itemProps, { children: jsx(RawField, Object.assign({}, fieldProps, { ref: ref })) })));
    });
    Object.keys(Field).forEach((key) => {
        if (!(key in ItemField)) {
            ItemField[key] = Field[key];
        }
    });
    return ItemField;
};

const { Item: Item$1 } = Form;
// https://github.com/react-component/field-form/blob/master/src/utils/typeUtil.ts
function toArray(value) {
    if (value === undefined || value === null) {
        return [];
    }
    return Array.isArray(value) ? value : [value];
}
// https://github.com/react-component/util/blob/master/src/utils/get.ts
function get(entity, path) {
    let current = entity;
    for (let i = 0; i < path.length; i += 1) {
        if (current === null || current === undefined) {
            return undefined;
        }
        current = current[path[i]];
    }
    return current;
}
const valOf = (values, name) => {
    const namePath = toArray(name);
    return get(values, namePath);
};
/**
 * Watch - 用于监听其它字段的值
 */
const Watch = (props) => {
    const { name, list, children, onlyValid = false, onChange } = props;
    const hasName = !Array.isArray(list);
    const prev = useRef(hasName ? undefined : list.map(() => undefined));
    return (jsx(Item$1, { noStyle: true, shouldUpdate: (prev, next) => {
            if (hasName) {
                return name ? valOf(prev, name) !== valOf(next, name) : true;
            }
            return list.some((key) => key && valOf(prev, key) !== valOf(next, key));
        }, children: (rawForm) => {
            const form = rawForm;
            const next = hasName
                ? name && form.getFieldValue(name)
                : list.map((key) => key && form.getFieldValue(key));
            let hasChange;
            let hasValidValue = false;
            let changeEffect = null;
            if (hasName) {
                hasChange = next !== prev.current;
                hasValidValue = next !== undefined;
            }
            else {
                hasChange = next.some((val, index) => {
                    var _a;
                    hasValidValue = val !== undefined;
                    return val !== ((_a = prev.current) === null || _a === void 0 ? void 0 : _a[index]);
                });
            }
            const cachedPrev = prev.current;
            if (hasChange) {
                prev.current = next;
                if (onChange) {
                    const ChangeEffect = () => {
                        useEffect(() => onChange(next, cachedPrev, form), []);
                        return null;
                    };
                    changeEffect = jsx(ChangeEffect, {});
                }
            }
            if (!onlyValid || (onlyValid && hasValidValue)) {
                return (jsxs(Fragment, { children: [children === null || children === void 0 ? void 0 : children(next, cachedPrev, form), changeEffect] }));
            }
            return changeEffect;
        } }));
};

const { Item } = Form;
const WrapperCol = (props) => {
    return jsx(Item, Object.assign({ label: jsx("span", {}), colon: false }, props));
};

const AutoComplete = create(AutoComplete$1);
const Cascader = create(Cascader$1);
const Checkbox = create(Checkbox$1);
const DatePicker = create(DatePicker$1);
const Input = create(Input$1);
const InputNumber = create(InputNumber$1);
const Mentions = create(Mentions$1);
const Radio = create(Radio$1);
const Rate = create(Rate$1);
const Select = create(Select$1);
const Slider = create(Slider$1);
const Switch = create(Switch$1);
const TimePicker = create(TimePicker$1);
const Transfer = create(Transfer$1);
const TreeSelect = create(TreeSelect$1);
const Upload = create(Upload$1);
const CheckboxGroup = create(Checkbox$1.Group);
const DateRange = create(DatePicker$1.RangePicker);
const TextArea = create(Input$1.TextArea);
const Search = create(Input$1.Search);
const Password = create(Input$1.Password);
const RadioGroup = create(Radio$1.Group);
const TimeRange = create(TimePicker$1.RangePicker);
const Dragger = create(Upload$1.Dragger);

export { AutoComplete, Cascader, Checkbox, CheckboxGroup, DatePicker, DateRange, Dragger, Input, InputNumber, Mentions, Password, Radio, RadioGroup, Rate, Search, Select, Slider, Switch, TextArea, TimePicker, TimeRange, Transfer, TreeSelect, Upload, Watch, WrapperCol, create };
